@@ -6,7 +6,7 @@ use Time::HiRes qw(sleep);
 use JSON::PP ();
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 my %COLLECTORS = (
     system  => \&_system_info,
@@ -16,17 +16,22 @@ my %COLLECTORS = (
     disk    => \&_disk_usage,
     disk_io => \&_disk_io,
     net     => \&_network_io,
+    process => \&_process_list,
 );
 
 sub collect_all {
-    return collect();
+    my ($options) = @_;
+    return collect(undef, $options);
 }
 
 sub collect {
-    my ($which) = @_;
+    my ($which, $options) = @_;
     my @names;
     if (!defined $which) {
         @names = qw(system cpu load mem disk disk_io net);
+        if ($options && ref $options eq 'HASH' && exists $options->{process}) {
+            push @names, 'process';
+        }
     } elsif (ref $which eq 'ARRAY') {
         @names = @$which;
     } else {
@@ -34,9 +39,10 @@ sub collect {
     }
 
     my %data = (timestamp => _timestamp());
+    my %collector_opts = ref $options eq 'HASH' ? %$options : ();
     for my $name (@names) {
         my $collector = $COLLECTORS{$name} or next;
-        my $value = eval { $collector->() };
+        my $value = eval { $collector->($collector_opts{$name}) };
         next if $@;
         $data{$name} = $value;
     }
@@ -257,6 +263,137 @@ sub _network_io {
     return \@ifaces;
 }
 
+sub _process_list {
+    my ($opts) = @_;
+    $opts ||= {};
+
+    my $uptime    = _uptime_seconds();
+    my $clk_tck   = POSIX::sysconf(&POSIX::_SC_CLK_TCK) || 100;
+    my $page_size = POSIX::sysconf(&POSIX::_SC_PAGESIZE) || 4096;
+    my $cores     = _cpu_cores() || 1;
+
+    opendir my $dh, '/proc' or return [];
+    my @pids = grep { /^\d+$/ } readdir $dh;
+    closedir $dh;
+
+    my @processes;
+    for my $pid (@pids) {
+        my $stat_path = "/proc/$pid/stat";
+        my $cmd_path  = "/proc/$pid/cmdline";
+        my $status_path = "/proc/$pid/status";
+        next unless -r $stat_path;
+
+        open my $sfh, '<', $stat_path or next;
+        my $stat_line = <$sfh> // '';
+        close $sfh;
+        next unless $stat_line =~ /^(\d+)\s+\((.*)\)\s+(.+)$/;
+
+        my ($parsed_pid, $comm, $rest) = ($1, $2, $3);
+        my @fields = split /\s+/, $rest;
+        next unless @fields >= 22;
+
+        my $state      = $fields[0];
+        my $ppid       = _maybe_number($fields[1]);
+        my $utime      = _maybe_number($fields[11]);
+        my $stime      = _maybe_number($fields[12]);
+        my $starttime  = _maybe_number($fields[19]);
+        my $rss_pages  = _maybe_number($fields[21]);
+        my $threads    = _maybe_number($fields[16]);
+
+        my $elapsed = defined $uptime && defined $starttime
+            ? $uptime - ($starttime / $clk_tck)
+            : undef;
+        my $cpu_pct =
+            defined $elapsed && $elapsed > 0 && defined $utime && defined $stime
+            ? sprintf('%.1f', (($utime + $stime) / $clk_tck) / $elapsed * 100 / $cores)
+            : 0;
+
+        my $rss_bytes = defined $rss_pages ? $rss_pages * $page_size : undef;
+
+        my $cmdline = '';
+        if (open my $cfh, '<', $cmd_path) {
+            local $/ = undef;
+            my $raw = <$cfh> // '';
+            close $cfh;
+            $raw =~ s/\0/ /g;
+            $cmdline = $raw;
+            $cmdline =~ s/\s+$//;
+        }
+
+        my $uid;
+        if (open my $stfh, '<', $status_path) {
+            while (my $line = <$stfh>) {
+                if ($line =~ /^Uid:\s+(\d+)/) {
+                    $uid = _maybe_number($1);
+                    last;
+                }
+            }
+            close $stfh;
+        }
+
+        push @processes, {
+            pid       => _maybe_number($parsed_pid),
+            ppid      => $ppid,
+            name      => $comm,
+            command   => length($cmdline) ? $cmdline : $comm,
+            state     => $state,
+            threads   => $threads,
+            uid       => $uid,
+            cpu_pct   => _maybe_number($cpu_pct),
+            rss_bytes => $rss_bytes,
+        };
+    }
+
+    my @watch = map { lc $_ } @{ $opts->{watch} // [] };
+    my $has_watch = @watch ? 1 : 0;
+
+    my @selected;
+    if ($has_watch) {
+        push @selected, grep { _process_matches($_, \@watch) } @processes;
+    }
+
+    if (my $n = $opts->{top_cpu}) {
+        push @selected, _top_processes(\@processes, 'cpu_pct', $n);
+    }
+
+    if (my $n = $opts->{top_rss}) {
+        push @selected, _top_processes(\@processes, 'rss_bytes', $n);
+    }
+
+    if (!$has_watch && !$opts->{top_cpu} && !$opts->{top_rss}) {
+        return \@processes;
+    }
+
+    my %seen;
+    my @deduped;
+    for my $proc (@selected) {
+        next if $seen{ $proc->{pid} }++;
+        push @deduped, $proc;
+    }
+
+    return \@deduped;
+}
+
+sub _top_processes {
+    my ($processes, $field, $n) = @_;
+    return () unless $processes && @$processes && $n && $field;
+    my @sorted = sort { ($b->{$field} // 0) <=> ($a->{$field} // 0) } @$processes;
+    my $limit = $n < @sorted ? $n : scalar @sorted;
+    return @sorted[0 .. $limit - 1];
+}
+
+sub _process_matches {
+    my ($proc, $watch) = @_;
+    return 0 unless $proc && $watch && @$watch;
+    my $name = lc($proc->{name} // '');
+    my $cmd  = lc($proc->{command} // '');
+    for my $needle (@$watch) {
+        return 1 if index($name, $needle) >= 0;
+        return 1 if index($cmd,  $needle) >= 0;
+    }
+    return 0;
+}
+
 sub _df_stats {
     open my $df, '-|', 'df', '-P', '-k' or return {};
     my %stats;
